import anywidget
import traitlets


class TableWidget(anywidget.AnyWidget):
    _esm = """
    function render({ model, el }) {
      let domElement = document.createElement("div");
      let innerHTML;
      el.classList.add("custom-table");
      let selectedIndex = -1; // Add a variable to keep track of the selected row index
      function drawTable() {
          const data = model.get("data");
          // clean up the dom element
          domElement.innerHTML = "";
          innerHTML = '<table><tr>' + data[0].map(header => `<th>${header}</th>`).join('') + '</tr>';
          for (let i = 1; i < data.length; i++) {
            innerHTML += '<tr>' + data[i].map(cell => `<td>${cell}</td>`).join('') + '</tr>';
          }
          innerHTML += "</table>"
          domElement.innerHTML += innerHTML
          // Adding click event listeners to each row
          const rows = domElement.querySelectorAll('tr');
          rows.forEach((row, index) => {
              // Skip the header row
              if (index > 0) {
                  row.addEventListener('click', () => {
                      // Remove selected class from previously selected row if any
                      if (selectedIndex >= 0 && rows[selectedIndex + 1]) {
                          rows[selectedIndex + 1].classList.remove('selected-row');
                      }
                      // Adjusted to match Python's 0-based indexing, subtract 1 to account for header
                      model.set('row_index', index - 1);
                      model.save_changes();
                      // Add selected class to the newly selected row
                      row.classList.add('selected-row');
                      selectedIndex = index - 1; // Update the selectedIndex
                  });
              }
          });
      }
      drawTable();
      domElement.addEventListener("row_click", () => {
        model.set("row_index", 1);
        model.save_changes();
      });
      model.on("change:data", () => {
        drawTable()
      });
      el.appendChild(domElement);
    }
    export default { render };
    """
    _css = """
    .custom-table table, .custom-table th, .custom-table td {
        border: 1px solid black;
        border-collapse: collapse;
        text-align: left;
        padding: 8px;
    }
    .custom-table th, .custom-table td {
        min-width: 60px;
        word-wrap: break-word;
    }
    .custom-table table {
        width: 100%;
        font-size: 1.2em;
    }
    /* hover effect */
    .custom-table tr:not(:first-child):hover, .custom-table tr.selected-row { /* Add .selected-row for persisting the background color */
        background-color: #FF474D; /* Light red background on hover and on selected row */
    }
    """
    data = traitlets.List().tag(sync=True)
    row_index = traitlets.Int().tag(sync=True)



import ipywidgets as ipw
from aiidalab_qe.common.panel import ResultPanel
from weas_widget import WeasWidget
from .widget import TableWidget


class Result(ResultPanel):
    title = "Bader Charge"
    workchain_labels = ["bader"]

    def __init__(self, node=None, **kwargs):
        super().__init__(node=node, **kwargs)
        self.summary_view = ipw.HTML()
        self.result_table = TableWidget()
        guiConfig = {
            "enabled": True,
            "components": {"atomsControl": True, "buttons": True},
            "buttons": {
                "fullscreen": True,
                "download": True,
                "measurement": True,
            },
        }
        self.structure_view = WeasWidget(guiConfig=guiConfig)
        self.result_table.observe(self._process_row_index, "row_index")
        self.structure_view_ready = False

    def _process_row_index(self, change):
        if change["new"] is not None:
            selected_atoms_indices = [self.result_table.data[change["new"] + 1][0]]
            self.structure_view.avr.selected_atoms_indices = selected_atoms_indices
            # trigger the resize event to update the view
            if not self.structure_view_ready:
                self.structure_view._widget.send_js_task(
                    {"name": "tjs.onWindowResize", "kwargs": {}}
                )
                self.structure_view._widget.send_js_task(
                    {
                        "name": "tjs.updateCameraAndControls",
                        "kwargs": {"direction": [0, -100, 0]},
                    }
                )
                self.structure_view_ready = True

    def _update_view(self):
        self.structure = self.node.inputs.bader.structure
        bader_charge = self.outputs.bader.bader.bader_charge.get_array("charge")
        self._update_structure(self.structure)
        self._generate_table(self.structure, bader_charge)
        table_help = ipw.HTML(
            """
            <div style='margin: 10px 0;'>
                <h4 style='margin-bottom: 5px; color: #3178C6;'>Result</h4>
            </div>
            """,
            layout=ipw.Layout(margin="0 0 20px 0"),  # Adjust the margin as needed
        )
        structure_help = ipw.HTML(
            """
            <div style='margin: 10px 0;'>
                <h4 style='margin-bottom: 5px; color: #3178C6;'>Structure</h4>
                <p style='margin: 5px 0; font-size: 14px;'>
                    Click on the row above to highlight the specific atom for which the Bader charge is being calculated.
                </p>
            </div>
            """,
            layout=ipw.Layout(margin="0 0 20px 0"),  # Adjust the margin as needed
        )
        self.children = [
            ipw.VBox(
                children=[
                    ipw.VBox([table_help, self.result_table]),
                    ipw.VBox([structure_help, self.structure_view]),
                ],
                layout=ipw.Layout(justify_content="space-between", margin="10px"),
            ),
        ]

    def _update_structure(self, structure):
        atoms = structure.get_ase()
        self.structure_view.from_ase(atoms)
        self.structure_view.avr.model_style = 1
        self.structure_view.avr.color_type = "VESTA"
        self.structure_view.avr.atom_label_type = "Index"

    def _generate_table(self, structure, bader_charge):
        # get index and element form AiiDA StructureData
        site_index = [site.kind_name for site in structure.sites]

        # Start of the HTML string for the table
        data = [["Site Index", "Element", "Bader Charge"]]
        # Add rows to the table based on the bader_charge
        for i in range(len(site_index)):
            charge = round(bader_charge[i], 2)
            data.append([i, site_index[i], charge])
        self.result_table.data = data



from aiidalab_qe.common.panel import OutlinePanel
from aiidalab_qe.common.widgets import (
    QEAppComputationalResourcesWidget as ComputationalResourcesWidget,
)

from .result import Result
from .workchain import workchain_and_builder


class BaderOutline(OutlinePanel):
    title = "Bader charge analysis"
    help = """"""


pp_code = ComputationalResourcesWidget(
    description="pp.x",
    default_calc_job_plugin="quantumespresso.pp",
)

bader_code = ComputationalResourcesWidget(
    description="bader",
    default_calc_job_plugin="bader",
)


bader = {
    "outline": BaderOutline,
    "code": {"pp": pp_code, "bader": bader_code},
    "result": Result,
    "workchain": workchain_and_builder,
}
